
        INTEGER FUNCTION IDMIN(N,X,INCX)
        INTEGER I,N,INCX
        DOUBLE PRECISION X(INCX,*),SMIN
C
C THIS FUNCTION RETURNS THE INDEX OF THE SMALLEST (ALGEBRAIC)
C COMPONENT OF X.  ADAPTED FROM PORT3 LIBRARY -- FEB 2008.
C ONLY EVERY INCXTH COMPONENT OF X IS CONSIDERED.
C
        IDMIN=0
        IF(N.EQ.0) RETURN
        SMIN=X(1,1)
        IDMIN=1
        DO 10 I=1,N
        IF(SMIN.LE.X(1,I)) GO TO 10
           SMIN=X(1,I)
           IDMIN=I
 10     CONTINUE
        RETURN
        END

